require 'spec_helper'

describe Lifen::Token do

  describe ':needs_to_be_refreshed?' do

    context 'default configuration' do
      it 'knows when a token needs to be refreshed' do
        token = Lifen::Token.new(value: "valid_token", expires_at: Time.now.to_i + 120)

        expect(token.needs_to_be_refreshed?).to be_falsy
      end

      it 'knows when a token needs to be refreshed' do
        token = Lifen::Token.new(value: "valid_token", expires_at: Time.now.to_i + 40)

        expect(token.needs_to_be_refreshed?).to be_falsy
      end
    end

    context 'custom configuration' do
      before do
        Lifen.configure do |config|
          config.expiration_margin = 60
        end
      end

      it 'knows when a token needs to be refreshed' do
        token = Lifen::Token.new(value: "valid_token", expires_at: Time.now.to_i + 120)

        expect(token.needs_to_be_refreshed?).to be_falsy
      end

      it 'knows when a token needs to be refreshed' do
        token = Lifen::Token.new(value: "valid_token", expires_at: Time.now.to_i + 40)

        expect(token.needs_to_be_refreshed?).to be_truthy
      end

      it 'refreshs the token' do
        token = Lifen::Token.new(value: "valid_token", expires_at: Time.now.to_i + 40)

        expect(token).to receive(:refresh) do
          token.expires_at = Time.now.to_i + 120
        end

        token.refresh_once_if_needed
      end
    end

  end

  describe 'expired token' do

    let(:expired_token)   { Lifen::Token.new(value: "valid_token", expires_at: Time.now.to_i - 60) }

    context ':refresh_once_if_needed' do
      it 'refreshs the token' do
        expect(expired_token).to receive(:refresh) do
          expired_token.expires_at = Time.now.to_i + 60
        end

        expired_token.refresh_once_if_needed
      end
    end

  end

  describe 'invalid token' do

    let(:invalid_token)   { Lifen::Token.new(value: "", expires_at: Time.now.to_i + 60) }

    it 'detects invalid tokens' do
      expect(invalid_token).to_not be_valid
    end

  end

  describe 'thread safe' do

    class TokenPersistedUser < Struct.new(:token_value, :token_expires_at) ; end

    let(:user)                    { Lifen::User.new }
    let(:persisted_user)          { TokenPersistedUser.new("expired_token", Time.now.to_i - 60) }
    let(:perstiting)              { double("Persisting") }

    let(:expired_token)           {
      Lifen::Token.new( user: user,
                        value: persisted_user.token_value,
                        expires_at: persisted_user.token_expires_at,
                        load_from_db: ->(token) {
                          # Loading data from DB

                          sleep 0.5

                          token.value = persisted_user.token_value
                          token.expires_at = persisted_user.token_expires_at
                        },
                        save_to_db: ->(token) {
                          # Saving data to DB

                          # ap "saving to db on expired token #{token.expires_at}"

                          persisted_user.token_value = token.value
                          persisted_user.token_expires_at = token.expires_at

                          perstiting.persist
                        }
                      )           }


    let(:other_expired_token)     {
      Lifen::Token.new( user: user,
                        value: persisted_user.token_value,
                        expires_at: persisted_user.token_expires_at,
                        load_from_db: ->(token) {
                          # Loading data from DB

                          sleep 0.5

                          token.value = persisted_user.token_value
                          token.expires_at = persisted_user.token_expires_at
                        },
                        save_to_db: ->(token) {
                          # Saving data to DB
                          # ap "saving to db on other expired token #{token.expires_at}"

                          persisted_user.token_value = token.value
                          persisted_user.token_expires_at = token.expires_at

                          perstiting.persist
                        }
                      )            }

    before do
      allow(expired_token).to receive(:refresh) do
        expired_token.value = "valid_token"
        expired_token.expires_at = Time.now.to_i + 180
      end

      allow(other_expired_token).to receive(:refresh) do
        other_expired_token.value = "valid_token"
        other_expired_token.expires_at = Time.now.to_i + 180
      end
    end

    15.times do
      it 'queues refresh calls' do
        expect(perstiting).to receive(:persist).once

        threads = []

        threads << Thread.new do
          expired_token.refresh_once_if_needed
        end

        # sleep 0.1 # Making sure the mocked call is received first

        threads << Thread.new do
          other_expired_token.refresh_once_if_needed
        end

        threads.map { |t| t.join } # waiting for all threads to finish
      end
    end

  end

end